<?php

declare(strict_types=1);

namespace FiloBlu\ApiExtension\Model;

use FiloBlu\ApiExtension\Api\CustomerServiceInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 *
 */
class CustomerService implements CustomerServiceInterface
{

    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    private $connection;
    /**
     * @var \Magento\Customer\Api\CustomerRepositoryInterface
     */
    private $customerRepository;

    /***
     * @param \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
     * @param \Magento\Framework\App\ResourceConnection $resourceConnection
     */
    public function __construct(CustomerRepositoryInterface $customerRepository, ResourceConnection $resourceConnection)
    {
        $this->connection = $resourceConnection->getConnection();
        $this->customerRepository = $customerRepository;
    }

    /**
     * @param string $phone
     * @return \Magento\Customer\Api\Data\CustomerInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function search(string $phone): CustomerInterface
    {
        $orderTable = $this->connection->getTableName('sales_order');
        $orderAddressTable = $this->connection->getTableName('sales_order_address');
        $customerAddressTable = $this->connection->getTableName('customer_address_entity');

        $select = $this->connection->select()->from(
            ['so' => $orderTable], ['customer_id']
        )->joinInner(['soa' => $orderAddressTable],
            'so.entity_id = soa.parent_id AND soa.telephone = :phone AND so.customer_id > 0',
            [])->group('so.customer_id');

        $customerId = (int)$this->connection->fetchOne($select, ['phone' => $phone]);

        if ($customerId > 0) {
            return $this->customerRepository->getById($customerId);
        }

        $select = $this->connection->select()->from(['cae' => $customerAddressTable], ['parent_id'])->where(
            'cae.telephone = :phone'
        );

        $customerId = (int) $this->connection->fetchOne($select, ['phone' => $phone]);

        if ($customerId > 0) {
            return $this->customerRepository->getById($customerId);
        }

        throw new NoSuchEntityException(__('Customer not found.'));
    }
}